# AutoBenchmark

Determine the speed of various AceSorting functions, for various array sizes.

**Version**: AceSorting v0.2

**DO NOT EDIT**: This file was auto-generated using `make README.md`.

## Dependencies

This program depends on the following libraries:

* [AceCommon](https://github.com/bxparks/AceCommon)

## How to Generate

This requires the [AUniter](https://github.com/bxparks/AUniter) script
to execute the Arduino IDE programmatically.

The `Makefile` has rules to generate the `*.txt` results file for several
microcontrollers that I usually support, but the `$ make benchmarks` command
does not work very well because the USB port of the microcontroller is a
dynamically changing parameter. I created a semi-automated way of collect the
`*.txt` files:

1. Connect the microcontroller to the serial port. I usually do this through a
USB hub with individually controlled switch.
2. Type `$ auniter ports` to determine its `/dev/ttyXXX` port number (e.g.
`/dev/ttyUSB0` or `/dev/ttyACM0`).
3. If the port is `USB0` or `ACM0`, type `$ make nano.txt`, etc.
4. Switch off the old microontroller.
5. Go to Step 1 and repeat for each microcontroller.

The `generate_table.awk` program reads one of `*.txt` files and prints out an
ASCII table that can be directly embedded into this README.md file. For example
the following command produces the table in the Nano section below:

```
$ ./generate_table.awk < nano.txt
```

Fortunately, we no longer need to run `generate_table.awk` for each `*.txt`
file. The process has been automated using the `generate_readme.py` script which
will be invoked by the following command:
```
$ make README.md
```

## CPU Time Changes

**v0.1**

* Initial results.
* The C-library `qsort()` is far slower than the C++ version because it uses
  a callback to a comparison function through a function pointer.

## Results

The following results show the runtime of each sorting function in milliseconds,
when sorting different array sizes.

### Arduino Nano

* 16MHz ATmega328P
* Arduino IDE 1.8.13
* Arduino AVR Boards 1.8.3
* `micros()` has a resolution of 4 microseconds

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.107 | 1.077 | 12.735 | 116.675 |         |         |
| insertionSort()     | 0.045 | 0.263 |  2.557 |  22.252 |         |         |
| selectionSort()     | 0.088 | 0.560 |  5.600 |  48.892 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.074 | 0.310 |  1.706 |   6.865 |         |         |
| shellSortKnuth()    | 0.101 | 0.330 |  1.450 |   5.665 |         |         |
| shellSortTokuda()   | 0.075 | 0.327 |  1.614 |   6.540 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.160 | 0.534 |  2.214 |   8.167 |         |         |
| combSort13m()       | 0.165 | 0.550 |  2.219 |   8.181 |         |         |
| combSort133()       | 0.086 | 0.396 |  1.944 |   7.702 |         |         |
| combSort133m()      | 0.086 | 0.416 |  1.978 |   7.740 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.096 | 0.368 |  1.568 |   5.651 |         |         |
| quickSortMedian()   | 0.117 | 0.431 |  1.717 |   5.905 |         |         |
| quickSortMdnSwppd() | 0.094 | 0.341 |  1.417 |   4.909 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.204 | 0.855 |  3.629 |  13.021 |         |         |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### SparkFun Pro Micro

* 16 MHz ATmega32U4
* Arduino IDE 1.8.13
* SparkFun AVR Boards 1.1.13
* `micros()` has a resolution of 4 microseconds

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.115 | 1.083 | 12.773 | 121.179 |         |         |
| insertionSort()     | 0.041 | 0.291 |  2.527 |  21.032 |         |         |
| selectionSort()     | 0.081 | 0.561 |  5.632 |  49.153 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.074 | 0.310 |  1.712 |   6.912 |         |         |
| shellSortKnuth()    | 0.101 | 0.339 |  1.445 |   5.601 |         |         |
| shellSortTokuda()   | 0.076 | 0.337 |  1.634 |   6.588 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.165 | 0.537 |  2.209 |   8.215 |         |         |
| combSort13m()       | 0.164 | 0.557 |  2.219 |   8.134 |         |         |
| combSort133()       | 0.086 | 0.389 |  1.996 |   7.767 |         |         |
| combSort133m()      | 0.091 | 0.423 |  1.997 |   7.729 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.099 | 0.375 |  1.619 |   5.711 |         |         |
| quickSortMedian()   | 0.119 | 0.427 |  1.748 |   5.976 |         |         |
| quickSortMdnSwppd() | 0.094 | 0.341 |  1.401 |   4.974 |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.200 | 0.839 |  3.661 |  12.974 |         |         |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### SAMD21 M0 Mini

* 48 MHz ARM Cortex-M0+
* Arduino IDE 1.8.13
* SparkFun SAMD Core 1.8.3

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.039 | 0.357 |  3.984 |  36.967 | 429.557 |         |
| insertionSort()     | 0.014 | 0.095 |  0.880 |   8.132 |  87.197 |         |
| selectionSort()     | 0.023 | 0.159 |  1.620 |  14.279 | 157.547 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.016 | 0.064 |  0.349 |   1.405 |   5.861 |  22.452 |
| shellSortKnuth()    | 0.019 | 0.067 |  0.289 |   1.153 |   5.111 |  19.448 |
| shellSortTokuda()   | 0.020 | 0.081 |  0.390 |   1.556 |   6.673 |  24.187 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.035 | 0.118 |  0.496 |   1.859 |   8.180 |  29.661 |
| combSort13m()       | 0.035 | 0.124 |  0.512 |   1.882 |   7.892 |  28.483 |
| combSort133()       | 0.020 | 0.090 |  0.454 |   1.764 |   7.897 |  26.989 |
| combSort133m()      | 0.021 | 0.097 |  0.454 |   1.752 |   7.623 |  27.649 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.021 | 0.073 |  0.315 |   1.077 |   4.211 |  14.031 |
| quickSortMedian()   | 0.026 | 0.088 |  0.344 |   1.161 |   4.343 |  14.455 |
| quickSortMdnSwppd() | 0.020 | 0.070 |  0.285 |   1.001 |   3.783 |  12.630 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.054 | 0.213 |  0.886 |   3.215 |  12.678 |  42.791 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### STM32

* STM32 "Blue Pill", STM32F103C8, 72 MHz ARM Cortex-M3
* Arduino IDE 1.8.13
* STM32duino 2.0.0

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.018 | 0.161 |  2.105 |  19.955 | 225.639 |         |
| insertionSort()     | 0.010 | 0.054 |  0.495 |   4.486 |  48.616 |         |
| selectionSort()     | 0.017 | 0.116 |  1.167 |  10.197 | 112.180 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.014 | 0.058 |  0.324 |   1.284 |   5.427 |  20.852 |
| shellSortKnuth()    | 0.014 | 0.059 |  0.305 |   1.236 |   5.604 |  21.566 |
| shellSortTokuda()   | 0.011 | 0.047 |  0.226 |   0.901 |   3.904 |  14.199 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.019 | 0.087 |  0.442 |   1.770 |   7.895 |  28.862 |
| combSort13m()       | 0.018 | 0.087 |  0.430 |   1.697 |   7.380 |  26.657 |
| combSort133()       | 0.018 | 0.085 |  0.417 |   1.649 |   7.630 |  26.148 |
| combSort133m()      | 0.019 | 0.089 |  0.419 |   1.619 |   7.151 |  25.380 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.019 | 0.067 |  0.278 |   0.990 |   3.835 |  13.145 |
| quickSortMedian()   | 0.020 | 0.076 |  0.311 |   1.101 |   4.212 |  14.381 |
| quickSortMdnSwppd() | 0.015 | 0.056 |  0.228 |   0.836 |   3.285 |  11.265 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.034 | 0.149 |  0.615 |   2.221 |   8.921 |  30.627 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### ESP8266

* NodeMCU 1.0 clone, 80MHz ESP8266
* Arduino IDE 1.8.13
* ESP8266 Boards 2.7.4

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.021 | 0.192 |  2.231 |  20.143 | 225.651 |         |
| insertionSort()     | 0.009 | 0.037 |  0.362 |   3.220 |  34.646 |         |
| selectionSort()     | 0.017 | 0.085 |  0.892 |   7.930 |  87.723 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.011 | 0.039 |  0.215 |   0.878 |   3.609 |  13.789 |
| shellSortKnuth()    | 0.010 | 0.036 |  0.172 |   0.690 |   3.022 |  11.304 |
| shellSortTokuda()   | 0.009 | 0.039 |  0.183 |   0.735 |   3.140 |  11.366 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.013 | 0.048 |  0.219 |   0.846 |   3.711 |  13.689 |
| combSort13m()       | 0.014 | 0.051 |  0.222 |   0.840 |   3.622 |  13.104 |
| combSort133()       | 0.010 | 0.040 |  0.208 |   0.796 |   3.571 |  12.430 |
| combSort133m()      | 0.010 | 0.044 |  0.206 |   0.793 |   3.465 |  12.387 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.013 | 0.045 |  0.185 |   0.651 |   2.519 |   8.307 |
| quickSortMedian()   | 0.016 | 0.052 |  0.200 |   0.677 |   2.551 |   8.403 |
| quickSortMdnSwppd() | 0.012 | 0.039 |  0.159 |   0.558 |   2.122 |   7.109 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.028 | 0.092 |  0.416 |   1.516 |   6.010 |  20.789 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### ESP32

* ESP32-01 Dev Board, 240 MHz Tensilica LX6
* Arduino IDE 1.8.13
* ESP32 Boards 1.0.6

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.006 | 0.049 |  0.622 |   5.704 |  63.678 |         |
| insertionSort()     | 0.004 | 0.017 |  0.157 |   1.295 |  15.081 |         |
| selectionSort()     | 0.005 | 0.032 |  0.341 |   3.035 |  33.618 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.004 | 0.014 |  0.077 |   0.302 |   1.283 |   4.861 |
| shellSortKnuth()    | 0.004 | 0.013 |  0.062 |   0.247 |   1.110 |   4.155 |
| shellSortTokuda()   | 0.004 | 0.014 |  0.065 |   0.263 |   1.115 |   4.034 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.005 | 0.019 |  0.095 |   0.377 |   1.688 |   6.257 |
| combSort13m()       | 0.005 | 0.020 |  0.097 |   0.373 |   1.641 |   5.996 |
| combSort133()       | 0.005 | 0.019 |  0.091 |   0.365 |   1.644 |   5.650 |
| combSort133m()      | 0.005 | 0.021 |  0.096 |   0.368 |   1.599 |   5.661 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.005 | 0.018 |  0.070 |   0.249 |   0.975 |   3.346 |
| quickSortMedian()   | 0.006 | 0.018 |  0.072 |   0.255 |   0.967 |   3.249 |
| quickSortMdnSwppd() | 0.004 | 0.014 |  0.059 |   0.211 |   0.826 |   2.803 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.008 | 0.032 |  0.143 |   0.524 |   2.099 |   7.221 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

### Teensy 3.2

* 96 MHz ARM Cortex-M4
* Arduino IDE 1.8.13
* Teensyduino 1.53
* Compiler options: "Faster"

```
+---------------------+------------------------+---------+---------+---------+
|            \      N |    10 |    30 |    100 |     300 |    1000 |    3000 |
| Function    \       |       |       |        |         |         |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| bubbleSort()        | 0.009 | 0.079 |  0.824 |   8.469 |  98.471 |         |
| insertionSort()     | 0.006 | 0.041 |  0.394 |   3.390 |  36.820 |         |
| selectionSort()     | 0.008 | 0.057 |  0.590 |   5.215 |  57.558 |         |
|---------------------+-------+-------+--------+---------+---------+---------|
| shellSortClassic()  | 0.007 | 0.029 |  0.162 |   0.648 |   2.712 |  10.394 |
| shellSortKnuth()    | 0.006 | 0.027 |  0.133 |   0.531 |   2.339 |   8.903 |
| shellSortTokuda()   | 0.007 | 0.028 |  0.136 |   0.539 |   2.314 |   8.366 |
|---------------------+-------+-------+--------+---------+---------+---------|
| combSort13()        | 0.008 | 0.036 |  0.182 |   0.722 |   3.233 |  11.887 |
| combSort13m()       | 0.008 | 0.038 |  0.186 |   0.725 |   3.139 |  11.431 |
| combSort133()       | 0.008 | 0.035 |  0.174 |   0.698 |   3.101 |  10.902 |
| combSort133m()      | 0.008 | 0.038 |  0.180 |   0.710 |   3.030 |  10.801 |
|---------------------+-------+-------+--------+---------+---------+---------|
| quickSortMiddle()   | 0.007 | 0.026 |  0.109 |   0.388 |   1.520 |   5.278 |
| quickSortMedian()   | 0.009 | 0.030 |  0.122 |   0.423 |   1.590 |   5.387 |
| quickSortMdnSwppd() | 0.007 | 0.025 |  0.100 |   0.365 |   1.412 |   4.752 |
|---------------------+-------+-------+--------+---------+---------+---------|
| qsort()             | 0.017 | 0.073 |  0.315 |   1.156 |   4.576 |  15.732 |
+---------------------+-------+-------+--------+---------+---------+---------+

```

